from __future__ import division

import gensim
import logging
import pysrt
from string import punctuation
import glob
import numpy
from scipy import spatial

from sklearn import svm
import matplotlib.pyplot as plt

import os

from nltk.corpus import sentiwordnet as swn
from nltk.corpus import wordnet as wn

import csv

import re
from nltk.compat import python_2_unicode_compatible
from nltk.corpus.reader import CorpusReader

from nltk.corpus import stopwords


logging.basicConfig(format='%(asctime)s : %(levelname)s : %(message)s', level=logging.INFO)

subtitleFolderName = "subtitles/"
testFolderName = "Test/"
labeledFolderName = "Emotion/"

subtitleFileNames = glob.glob(subtitleFolderName+"T*.srt")
testFileNames = glob.glob(testFolderName+"T*.srt")

# emotionFileNames = glob.glob(labeledFolderName+"Titanic.emotion")

# modelPath = "/tmp/"
# modelFileName = "word2vecModel_on_subtitles"

modelPath = "/home/uutsav/cs365/project/Model/utsav/"
modelFileName = "vec_model"

model = ""

majorEmo = ["love", "happy", "surprise", "sad", "disgust",  "anger", "fear"]
# not including emotionless as its vector won't be searched

testEmo = []
for emo in majorEmo:
    testEmo.append(emo)
testEmo.append("emotionless")

# print "majorEmo: ", majorEmo
# print "testEmo: ", testEmo

# reordering for printing and creating plots

majorEmotionsForStringForComparison = [ "love", "happy", "surprise", "emotionless", "sad", "disgust", "anger", "fear"]

testEmo_forSVM = [ "Love", "Happiness", "Surprise", "Emotionless", "Sad", "Disgust", "Anger", "Fear"]


ifRemoveStopWords = True
ifUseSentiWordNet = False
sentiWordNet_ThreshHold = 0.2
sentiWordNet_majorEmo_ThreshHold = 0.1

ifReweight_using_sentiWordNet = True

import numpy as np
import math


class Emotion:

    emoCount = 0

    def __init__(self, name):
        self.name = name
        Emotion.emoCount += 1
        self.true_pos = 0
        self.true_neg = 0
        self.false_pos = 0
        self.false_neg = 0
        self.accuracy = -1.0
        self.precision = -1.0

    def printObj(self):
        if self.true_pos+self.true_neg > 0:
            self.accuracy = self.true_pos/(self.true_pos+self.true_neg)

        if self.true_pos+self.false_pos > 0:
            self.precision = self.true_pos/(self.true_pos+self.false_pos)

        # print self.true_pos, "/", self.true_pos+self.true_neg, "= ", self.true_pos/(self.true_pos+self.true_neg), " = ", self.accuracy
        # print self.true_pos, "/", self.true_pos+self.false_pos, "= ", self.true_pos/(self.true_pos+self.false_pos), " = ", self.precision


        print "Emotion: ", self.name
        print "True Positive: ", self.true_pos, "\t", "True negative: ", self.true_neg
        print "False Positive: ", self.false_pos, "\t", "False negative: ", self.false_neg
        print "Accuracy: ", 100.0*self.accuracy, "%\t", "Precision: ", 100.0*self.precision, "%"

    def clear_stats(self):
        self.true_pos = 0
        self.true_neg = 0
        self.false_pos = 0
        self.false_neg = 0
        self.accuracy = -1.0
        self.precision = -1.0



emoObjs = {}
for emo in testEmo:
    emoObj = Emotion(emo)
    emoObjs[emo] = emoObj


def parse(subtitleFileName):

    subtitleText = []
    try:
        subtitleLines = pysrt.open(subtitleFileName, encoding='utf-8')
        # print "SRT contains: ", subtitleLines
        for i in range(len(subtitleLines)):
            # print subtitleLines[i].text
            lineNoPunct = ' '.join(filter(None, (word.strip(punctuation) for word in subtitleLines[i].text.split() )))
            allWords = lineNoPunct.split()
            allWords = [ x.lower() for x in allWords ]
            # print "wwwwww ", allWords
            subtitleText.append(allWords)

        # print subtitleText
        return subtitleText
    except:
        return 0


def load_model():
    global model
    model = gensim.models.Word2Vec.load(modelPath+modelFileName)


def build_model():

    allSentences = []
    skipCount = 0
    doneCount = 0

    # Word2vec doesn't allow to add new words to its vocabulary after its model is made the first time
    # So loading model and training on it is utterly useless as it won't add new words

    for subtitleFileName in subtitleFileNames:
        # print "filename: ", subtitleFileName
        sentences = parse(subtitleFileName)
        if sentences == 0:
            print subtitleFileName, " skipped"

            os.remove(subtitleFileName)  # remove then so they not pester afterwards

            skipCount += 1
            continue
        # print subtitleFileName, " done"
        allSentences += sentences

        # if doneCount == 0:
        #     model = gensim.models.Word2Vec(allSentences, min_count=5, size=200)
        #     model.save(modelPath+modelFileName)
        #
        # else:
        #     model = gensim.models.Word2Vec.load(modelPath+modelFileName)
        #     model.build_vocab(allSentences)
        #     model.train(allSentences)
        #     model.save(modelPath+modelFileName)

        doneCount += 1

    model = gensim.models.Word2Vec(allSentences, min_count=5, size=200)
    model.save(modelPath+modelFileName)

    # print doneCount, " files analysed"
    # print skipCount, " files skipped"


def remove_stopwords_from_line(line):

    stops = set(stopwords.words("english"))
    # print "Stop Words: ", stops

    # print "Subtitle Line in stopwords: ", line

    lineNoPunct = ' '.join(filter(None, (word.strip(punctuation) for word in line.split())))

    allWords = lineNoPunct.split()
    allWords = [ x.lower() for x in allWords ]

    # print "before removing stopwords: ", allWords
    allWords = [w for w in allWords if not w in stops]
    # print "after removing stopwords: ", allWords

    return allWords


def remove_nonSentiWord( word ):

    shouldInclude = False
    if len(list(swn.senti_synsets(word))) == 0:
        # print word, " not in sentiWordNet"
        if len(list(wn.synsets(word))) == 0:
            # print word, " not in wordNet"
            shouldInclude = False
        else:
            shouldInclude = True
    else:
        synSet = list(swn.senti_synsets(word))
        # print "Word: ", word
        for item in synSet:
            # print "+ ", item.pos_score(), " - ", item.neg_score(), " Neutral ", item.obj_score()
            if item.pos_score() > sentiWordNet_ThreshHold or item.neg_score() > sentiWordNet_ThreshHold:
                shouldInclude = True
                break
    return shouldInclude


def get_net_pos_neg(word):
    netPos = 0
    netNeg = 0

    if len(list(swn.senti_synsets(word))) != 0:
        sentisyn = list(swn.senti_synsets(word))
        for item in sentisyn:
            netPos += item.pos_score()
            netNeg += item.neg_score()

    return netPos, netNeg



def reweight_emo_vector(word):

    netPos = 0
    netNeg = 0
    netPos, netNeg = get_net_pos_neg(word)
    # print "word: ", word, "netpos: ", netPos, " netNeg: ", netNeg

    emoVec = numpy.array(model[word])

    if netNeg > sentiWordNet_majorEmo_ThreshHold or netPos > sentiWordNet_majorEmo_ThreshHold: # this sentiment is emotional

        if netNeg > netPos:
            polarity = -1
        else:
            polarity = +1

        # print "polarity: ", polarity

        factor_sum = 0
        word_synset = list(wn.synsets(word))
        for item in word_synset:
            synonym = item.name().split(".")[0]

            if synonym == word:
                # print "synonym same as word"
                continue

            synonym_netPos, synonym_netNeg = get_net_pos_neg(synonym)

            # print "synonym ", synonym, " has +", synonym_netPos, " and -", synonym_netNeg

            if polarity == 1:
                factor = synonym_netPos
            else:
                factor = synonym_netNeg

            # factor *= 10

            try:
                synonym_vec = numpy.array(model[synonym])
                synonym_vec = synonym_vec*factor

                if factor_sum == 0:
                    synonym_sumvec = synonym_vec
                else:
                    synonym_sumvec += synonym_vec

                factor_sum += factor

                if factor > 0:
                    print synonym, " brings a change to ", word

            except:
                print synonym, " not in vocab"

        if factor_sum == 0:
            return emoVec

        emoVec = (emoVec + synonym_sumvec)/(factor_sum+1)

    return emoVec




def test(subtitleFileName, trueEmotions):

    subtitleText = []

    # print "TrueEmotions: ", trueEmotions
    # return
    labeledEmotions = []

    majorEmoReallyUsed = []
    emoVecs = []
    emotionlessCount = 0

    for word in majorEmo:
        # try:
        emoVec = model[word]
        emoVec = numpy.array(emoVec)
        # print word, " ", emoVec

        if ifReweight_using_sentiWordNet == True:
            emoVec_new = reweight_emo_vector(word)

            emoDis = spatial.distance.cosine(emoVec, emoVec_new)
            print word, "cosine dis: ", emoDis
            # print word, " ", emoVec_new



        emoVecs.append(emoVec)
        majorEmoReallyUsed.append(word)
        # except:
        #     print word, " emotion skipped"


    # print "Major Emotions Really used: ", majorEmoReallyUsed
    emoCount = [0]*len(majorEmoReallyUsed)

    print model
    # try:
    subtitleLines = pysrt.open(subtitleFileName, encoding='utf-8')

    allsenVec = []
    y = []

    # print "SRT contains: ", subtitleLines
    for i in range(len(subtitleLines)):

        if ifRemoveStopWords == True:
            allWords = remove_stopwords_from_line(subtitleLines[i].text)

        else:
            lineNoPunct = ' '.join(filter(None, (word.strip(punctuation) for word in subtitleLines[i].text.split())))

            allWords = lineNoPunct.split()
            allWords = [ x.lower() for x in allWords ]

        subtitleText.append(allWords)

        usedWords = []
        numWords = 0
        for word in allWords:

            if ifUseSentiWordNet == True:
                if remove_nonSentiWord(word) == False:  # it is not a sentiword
                    # print word, " is not senti"
                    continue

            try:
                wordVec = model[word]
                wordVec = numpy.array(wordVec)

                if numWords >= 1:
                    senVec += wordVec
                else:
                    senVec = wordVec
                numWords += 1
                usedWords.append(word)
                # print word, " wordVec ", wordVec, " senVec", senVec
            except:
                # print word, " skipped"
                continue



        if numWords == 0:
            emotionlessCount += 1
            labeledEmotions.append(len(majorEmoReallyUsed))
            # if "emotionless" == majorEmotionsForStringForComparison[trueEmotions[i]]:
            print "1#", i, " ", subtitleLines[i].text, " emo: ", "Emotionless", " truth: ", majorEmotionsForStringForComparison[trueEmotions[i]]
            # print "Line: #", i, " ", subtitleLines[i].text, " Left: ", usedWords
            continue

        senVec = senVec/numWords
        allsenVec.append(senVec)
        y.append(trueEmotions[i])

        # COSINE DISTANCE

        # The resulting similarity from -1 meaning exactly opposite
        # to 1 meaning exactly the same, with 0 indicating orthogonality (decorrelation)
        #
        # But since spatial uses 1 - (a.b)/|a|.|b| for claculating cosine distance instead of (a.b)/|a|.|b|
        # so a value of:
        # 0 means exactly the same
        # 1 means decorrelation
        # 2 means exactly opposite

        minDis = 1
        count = 0

        for emoVec in emoVecs:

            # print "Emo: ", majorEmoReallyUsed[count]
            # EUCLEDIAN DISTANCE
            # emoDis = numpy.linalg.norm((emoVec - senVec), ord=2)
            # print "Euclid dis: ", emoDis

            # COSINE DISTANCE
            emoDis = spatial.distance.cosine(emoVec, senVec)
            # print "Cosine Dis: ", emoDis, " with emo: ", majorEmo[count], " min dis: ", minDis

            if emoDis < minDis:
                bestEmo = count
                minDis = emoDis
            count += 1

        if minDis < 1:
            emoCount[bestEmo] += 1
            labeledEmotions.append(bestEmo)
            # if majorEmoReallyUsed[bestEmo] == majorEmotionsForStringForComparison[trueEmotions[i]]:# and ( majorEmotionsForStringForComparison[trueEmotions[i]] == "anger" or  testEmo[trueEmotions[i]] == "disgust" ):
            print "2#", i, " ", subtitleLines[i].text, " Left: ", usedWords,  " emo: ", majorEmoReallyUsed[bestEmo], " truth: ", majorEmotionsForStringForComparison[trueEmotions[i]]
        else:
            emotionlessCount += 1
            labeledEmotions.append(len(majorEmoReallyUsed))
            # if "emotionless" == majorEmotionsForStringForComparison[trueEmotions[i]]:# and ( majorEmotionsForStringForComparison[trueEmotions[i]] == "anger" or  majorEmotionsForStringForComparison[trueEmotions[i]] == "disgust" ):
            print "3#", i, " ", subtitleLines[i].text, " Left: ", usedWords, " emo: ", "Emotionless", " truth: ", majorEmotionsForStringForComparison[trueEmotions[i]]

        # if i > 1000:
        #     break

    # print subtitleText

    print "Number of Dialogues: ", len(y)

    for i in range(len(majorEmoReallyUsed)):
        print majorEmoReallyUsed[i], " ", emoCount[i]

    print "Emotionless ", emotionlessCount

    for emo in majorEmotionsForStringForComparison:
        emoObjs[emo].clear_stats()


    for i in range(len(labeledEmotions)):
        # print subtitleLines[i].text
        # print "label: ", labeledEmotions[i], " and truth: ", trueEmotions[i]
        if labeledEmotions[i] == trueEmotions[i]:
            emo = majorEmotionsForStringForComparison[trueEmotions[i]]
            # print "true pos: ", emo
            emoObjs[emo].true_pos += 1
        else:
            emo_false_pos = majorEmotionsForStringForComparison[labeledEmotions[i]]
            # print "false pos: ", emo_false_pos
            emo_true_neg = majorEmotionsForStringForComparison[trueEmotions[i]]
            # print "true neg: ", emo_true_neg
            emoObjs[emo_true_neg].true_neg += 1
            emoObjs[emo_false_pos].false_pos += 1

    for emo in majorEmotionsForStringForComparison:
        emoObjs[emo].printObj()

    y_in_words = []     # y contains the ground truth emotions, converting it to words
    for item in y:
        y_in_words.append(majorEmotionsForStringForComparison[item])    # using majorEmotionsForStringForComparison to also map emotionless as it is not in majorEmo

    y_for_SVM = []  # some dialogues may not reach here since they were eliminated due to srt parse error, no sentiment word etc
    for item in y_in_words:
        counter = 0
        for word in majorEmotionsForStringForComparison:
            if word == item:
                y_for_SVM.append(counter)
                break
            counter += 1


    with open("output.csv", "wb") as f:
        writer = csv.writer(f)
        writer.writerows(allsenVec)

    file = open("y_in_words.txt", 'w')
    for item in y_in_words:
        file.write("%s\n" % item)


    file = open("y.txt", 'w')
    for item in y_for_SVM:
        file.write("%s\n" % item)


    # lin_clf = svm.SVC()
    # lin_clf.fit(allsenVec, y_for_SVM)

    # print "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAa"
    # nn = NeuralNetwork([200, 200, 1])
    #
    # nn.fit(np.array(allsenVec), y)
    #
    # # print y
    # print "QQQQQQQQQQQQQQQQQQQQQQQQQQQQQQ"
    # for e in np.array(allsenVec):
    #     print(nn.predict(e))
    # print "ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ"

    # accuracy_arr = []
    # max_accuracy = 0
    # for emo in testEmo_forSVM:
    #     accuracy_arr.append(emoObjs[emo].accuracy)
    #     max_accuracy = max(max_accuracy, emoObjs[emo].accuracy)
    #
    #
    #
    # # min_acc = min(accuracy_arr)
    #
    #
    #
    # print "accuracy array: ", accuracy_arr
    # print "max accuracy: ", max_accuracy
    # xmin = 0
    # xmax = len(testEmo_forSVM)
    # ymin = 0.1
    # ymax = max_accuracy + 0.10
    #
    # X = range(len(testEmo_forSVM))
    # Y = accuracy_arr
    # my_xticks = testEmo_forSVM
    # plt.xticks(X, my_xticks)
    # plt.plot(X, Y)
    #
    # plt.title('')
    # plt.ylabel('Accuracy')
    # plt.xlabel("Emotions")
    # plt.xlim([xmin, xmax])
    # plt.ylim([ymin, ymax])
    #
    # overall_acc = 0
    # for emo in testEmo:
    #     overall_acc += emoObjs[emo].true_pos
    # # plt.margins(tight=False)
    #
    # overall_acc = overall_acc/2046
    # print overall_acc, "%"
    #
    # plt.show()

    return labeledEmotions

    # except:
    #     return 0


def get_truthEmotions( fileName ):
    file = open(labeledFolderName+fileName, 'r')
    trueEmotions = file.read().splitlines()
    trueEmotions = list(map(int, trueEmotions))
    return trueEmotions


def test_on_labeled_data():
    for subtitleFileName in testFileNames:
        print "filename: ", subtitleFileName

        testfileName_without_extension = subtitleFileName.split(".")[-2].split("/")[-1]
        # print "testfileName_without_extension: ", testfileName_without_extension

        trueEmotions = get_truthEmotions(testfileName_without_extension+".emotion")

        # print "True Emotions: ", trueEmotions

        labeledEmotions = test(subtitleFileName, trueEmotions)
        if labeledEmotions == 0:
            print subtitleFileName, " skipped while testing"
            continue


def test_sentiWordNet():

    # testWords = ["good", "luxurious", "presumptous", "uncouth", "vulgar", "pitiful", "delightful", "remarkable", "splendid"]

    # testWords = ["happy", "fear", "surprise", "anger", "disgust", "sad"]

    testWords = ["absurd", "moron", "fun", "good", "disgust", "sad", "slut", "idiot"]

    for word in testWords:
        # if len(list(swn.senti_synsets(word))) == 0:
        #     print word, " not in sentiWordNet"
        # else:
        synSet = list(swn.senti_synsets(word))
        print "Word: ", word
        for item in synSet:
            print item
            print "+ ", item.pos_score(), " - ", item.neg_score(), " Neutral ", item.obj_score()

        synSet = list(wn.synsets(word))

        print "Word: ", word
        for item in synSet:
            print item.name().split(".")[0]



def main():

    # build_model()

    load_model()

    # for emo in majorEmo:
    #     print "emo: ", emo
    #     print model[emo]

    test_on_labeled_data()

    # test_sentiWordNet()

    #
    #
    # # min_acc = min(accuracy_arr)
    #
    # max_accuracy = max(accuracy_arr)
    # min_accuracy = min(accuracy_arr)
    #
    # print "accuracy array: ", accuracy_arr
    # print "max accuracy: ", max_accuracy
    # xmin = 0
    # xmax = len(testEmo_forSVM)-1
    # ymin = min_accuracy - 0.05
    # ymax = max_accuracy + 0.10
    #
    # X = range(len(testEmo_forSVM))
    # Y = accuracy_arr
    # my_xticks = testEmo_forSVM
    # plt.xticks(X, my_xticks)
    # plt.plot(X, Y)
    #
    # plt.title('Emotion-wise split of Accuracy using SVM')
    # plt.ylabel('Accuracy')
    # plt.xlabel("Emotions")
    # plt.xlim([xmin, xmax])
    # plt.ylim([ymin, ymax])
    #
    # # overall_acc = 0
    # # for emo in testEmo:
    # #     overall_acc += emoObjs[emo].true_pos
    # # plt.margins(tight=False)
    #
    # # overall_acc = overall_acc/2046
    # # print overall_acc, "%"
    #
    # plt.show()


main()
















